# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_delete_operating_system import GmpDeleteOperatingSystemTestMixin
from .test_get_operating_system import GmpGetOperatingSystemTestMixin
from .test_get_operating_systems import GmpGetOperatingSystemsTestMixin
from .test_modify_operating_system import GmpModifyOperatingSystemTestMixin

__all__ = (
    "GmpDeleteOperatingSystemTestMixin",
    "GmpGetOperatingSystemTestMixin",
    "GmpGetOperatingSystemsTestMixin",
    "GmpModifyOperatingSystemTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetAgentInstallerFileTestMixin:
    def test_get_agent_installer(self):
        self.gmp.get_agent_installer_file(agent_installer_id="installer1")

        self.connection.send.has_been_called_with(
            b'<get_agent_installer_file agent_installer_id="installer1"/>'
        )

    def test_get_agent_installer_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_agent_installer_file(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_agent_installer_file("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .secinfo import (
    GmpGetCertBundListTestMixin,
    GmpGetCertBundTestMixin,
    GmpGetCpeListTestMixin,
    GmpGetCpeTestMixin,
    GmpGetCveListTestMixin,
    GmpGetCveTestMixin,
    GmpGetDfnCertListTestMixin,
    GmpGetDfnCertTestMixin,
    GmpGetInfoListTestMixin,
    GmpGetInfoTestMixin,
    GmpGetNvtFamiliesTestMixin,
    GmpGetNvtListTestMixin,
    GmpGetNvtTestMixin,
    GmpGetScanConfigNvtsTestMixin,
    GmpGetScanConfigNvtTestMixin,
)


class Gmpv224GetCertBundTestCase(GmpGetCertBundTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetCpeTestCase(GmpGetCpeTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetCveTestCase(GmpGetCveTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetDfnCertCase(GmpGetDfnCertTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetInfoListTestCase(GmpGetInfoListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetInfoTestCase(GmpGetInfoTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetNvtTestCase(GmpGetNvtTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetScanConfigNvtTestCase(
    GmpGetScanConfigNvtTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetNvtFamiliesTestCase(
    GmpGetNvtFamiliesTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetScanConfigNvtsTestCase(
    GmpGetScanConfigNvtsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetCertBundListTestCase(
    GmpGetCertBundListTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetCpeListTestCase(GmpGetCpeListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetCveListTestCase(GmpGetCveListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetDfnCertListCase(GmpGetDfnCertListTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetNvtListTestCase(GmpGetNvtListTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .operating_systems import (
    GmpDeleteOperatingSystemTestMixin,
    GmpGetOperatingSystemsTestMixin,
    GmpGetOperatingSystemTestMixin,
    GmpModifyOperatingSystemTestMixin,
)


class Gmpv224DeleteOperatingSystemTestCase(
    GmpDeleteOperatingSystemTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetOperatingSystemTestCase(
    GmpGetOperatingSystemTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetOperatingSystemsTestCase(
    GmpGetOperatingSystemsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyOperatingSystemTestCase(
    GmpModifyOperatingSystemTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.port_lists import (
    GmpClonePortListTestMixin,
    GmpCreatePortListTestMixin,
    GmpCreatePortRangeTestMixin,
    GmpDeletePortListTestMixin,
    GmpDeletePortRangeTestMixin,
    GmpGetPortListsTestMixin,
    GmpGetPortListTestMixin,
    GmpModifyPortListTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225ClonePortListTestCase(GmpClonePortListTestMixin, GMPTestCase):
    pass


class Gmpv225CreatePortListTestCase(GmpCreatePortListTestMixin, GMPTestCase):
    pass


class Gmpv225CreatePortRangeListTestCase(
    GmpCreatePortRangeTestMixin, GMPTestCase
):
    pass


class Gmpv225DeletePortListTestCase(GmpDeletePortListTestMixin, GMPTestCase):
    pass


class Gmpv225DeletePortRangeTestCase(GmpDeletePortRangeTestMixin, GMPTestCase):
    pass


class Gmpv225GetPortListTestCase(GmpGetPortListTestMixin, GMPTestCase):
    pass


class Gmpv225GetPortListsTestCase(GmpGetPortListsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyPortListTestCase(GmpModifyPortListTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""Latest supported protocols, including unstable ones.

This module exposes the latest supported protocols of GVM including versions
not yet released as stable, which may contain features that can change at any time
and may not be available in all builds of the gvmd back-end.

The provided Gmp class implements the latest Greenbone Management Protocol.
The provided Osp class implements the latest Open Scanner Protocol.

For details about the possible supported protocol versions please take a look at
:py:mod:`gvm.protocols`.

Exports:
  - :py:class:`gvm.protocols.gmp.GMPNext`
  - :py:class:`gvm.protocols.ospv1.Osp`
"""

from .gmp import (
    GMPNext as GMP,  # noqa: N814
)
from .ospv1 import Osp

Gmp = GMP  # for backwards compatibility

__all__ = [
    "GMP",
    "Gmp",
    "Osp",
]



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
Greenbone Management Protocol (GMP) version 22.5
"""

from .._protocol import T
from ._gmp224 import GMPv224
from .requests.v225 import (
    ResourceNames,
    ResourceType,
)


class GMPv225(GMPv224[T]):
    """
    A class implementing the Greenbone Management Protocol (GMP) version 22.5

    Example:

        .. code-block:: python

            from gvm.protocols.gmp import GMPv225 as GMP

            with GMP(connection) as gmp:
                resp = gmp.get_tasks()
    """

    @staticmethod
    def get_protocol_version() -> tuple[int, int]:
        return (22, 5)

    def get_resource_names(
        self,
        resource_type: ResourceType,
        *,
        filter_string: str | None = None,
    ) -> T:
        """Request a list of resource names and IDs

        Arguments:
            resource_type: Type must be either ALERT, CERT_BUND_ADV,
                CONFIG, CPE, CREDENTIAL, CVE, DFN_CERT_ADV, FILTER,
                GROUP, HOST, NOTE, NVT, OS, OVERRIDE, PERMISSION,
                PORT_LIST, REPORT_FORMAT, REPORT, RESULT, ROLE,
                SCANNER, SCHEDULE, TARGET, TASK, TLS_CERTIFICATE
                or USER
            filter_string: Filter term to use for the query
        """
        return self._send_request_and_transform_response(
            ResourceNames.get_resource_names(
                resource_type, filter_string=filter_string
            )
        )

    def get_resource_name(
        self, resource_id: str, resource_type: ResourceType
    ) -> T:
        """Request a single resource name

        Arguments:
            resource_id: ID of an existing resource
            resource_type: Type must be either ALERT, CERT_BUND_ADV,
                CONFIG, CPE, CREDENTIAL, CVE, DFN_CERT_ADV, FILTER,
                GROUP, HOST, NOTE, NVT, OS, OVERRIDE, PERMISSION,
                PORT_LIST, REPORT_FORMAT, REPORT, RESULT, ROLE,
                SCANNER, SCHEDULE, TARGET, TASK, TLS_CERTIFICATE
                or USER
        """
        return self._send_request_and_transform_response(
            ResourceNames.get_resource_name(resource_id, resource_type)
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetTagTestMixin:
    def test_get_tag(self):
        self.gmp.get_tag("t1")

        self.connection.send.has_been_called_with(b'<get_tags tag_id="t1"/>')

        self.gmp.get_tag(tag_id="t1")

        self.connection.send.has_been_called_with(b'<get_tags tag_id="t1"/>')

    def test_get_tag_missing_tag_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_tag(tag_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_tag("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .tags import (
    GmpCloneTagTestMixin,
    GmpCreateTagTestMixin,
    GmpDeleteTagTestMixin,
    GmpGetTagsTestMixin,
    GmpGetTagTestMixin,
    GmpModifyTagTestMixin,
)


class Gmpv224DeleteTagTestCase(GmpDeleteTagTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTagTestCase(GmpGetTagTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTagsTestCase(GmpGetTagsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneTagTestCase(GmpCloneTagTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateTagTestCase(GmpCreateTagTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyTagTestCase(GmpModifyTagTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import PortRangeType


class GetPortRangeTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            PortRangeType.from_string("foo")

    def test_none_or_empty(self):
        ct = PortRangeType.from_string(None)
        self.assertIsNone(ct)
        ct = PortRangeType.from_string("")
        self.assertIsNone(ct)

    def test_tcp(self):
        ct = PortRangeType.from_string("tcp")
        self.assertEqual(ct, PortRangeType.TCP)

    def test_udp(self):
        ct = PortRangeType.from_string("udp")
        self.assertEqual(ct, PortRangeType.UDP)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import ReportFormats, ReportFormatType


class ReportFormatsTestCase(unittest.TestCase):
    def test_clone_report_format(self):
        request = ReportFormats.clone_report_format("report_format_id")
        self.assertEqual(
            bytes(request),
            b"<create_report_format>"
            b"<copy>report_format_id</copy>"
            b"</create_report_format>",
        )

        request = ReportFormats.clone_report_format(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b"<create_report_format>"
            b"<copy>c402cc3e-b531-11e1-9163-406186ea4fc5</copy>"
            b"</create_report_format>",
        )

    def test_clone_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            ReportFormats.clone_report_format(None)

        with self.assertRaises(RequiredArgument):
            ReportFormats.clone_report_format("")

    def test_delete_report_format(self):
        request = ReportFormats.delete_report_format("report_format_id")
        self.assertEqual(
            bytes(request),
            b'<delete_report_format report_format_id="report_format_id" ultimate="0"/>',
        )

        request = ReportFormats.delete_report_format(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b'<delete_report_format report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5" ultimate="0"/>',
        )

    def test_delete_report_format_with_ultimate(self):
        request = ReportFormats.delete_report_format(
            "report_format_id", ultimate=True
        )
        self.assertEqual(
            bytes(request),
            b'<delete_report_format report_format_id="report_format_id" ultimate="1"/>',
        )

        request = ReportFormats.delete_report_format(
            ReportFormatType.PDF, ultimate=False
        )
        self.assertEqual(
            bytes(request),
            b'<delete_report_format report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5" ultimate="0"/>',
        )

    def test_delete_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            ReportFormats.delete_report_format(None)

        with self.assertRaises(RequiredArgument):
            ReportFormats.delete_report_format("")

    def test_get_report_formats(self):
        request = ReportFormats.get_report_formats()
        self.assertEqual(
            bytes(request),
            b"<get_report_formats/>",
        )

    def test_get_report_formats_with_filter_string(self):
        request = ReportFormats.get_report_formats(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_report_formats filter="filter_string"/>',
        )

    def test_get_report_formats_with_filter_id(self):
        request = ReportFormats.get_report_formats(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_report_formats filt_id="filter_id"/>',
        )

    def test_get_report_formats_with_details(self):
        request = ReportFormats.get_report_formats(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats details="1"/>',
        )

        request = ReportFormats.get_report_formats(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats details="0"/>',
        )

    def test_get_report_formats_with_alerts(self):
        request = ReportFormats.get_report_formats(alerts=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats alerts="1"/>',
        )

        request = ReportFormats.get_report_formats(alerts=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats alerts="0"/>',
        )

    def test_get_report_formats_with_params(self):
        request = ReportFormats.get_report_formats(params=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats params="1"/>',
        )

        request = ReportFormats.get_report_formats(params=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats params="0"/>',
        )

    def test_get_report_formats_with_trash(self):
        request = ReportFormats.get_report_formats(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats trash="1"/>',
        )

        request = ReportFormats.get_report_formats(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats trash="0"/>',
        )

    def test_get_report_format(self):
        request = ReportFormats.get_report_format("report_format_id")
        self.assertEqual(
            bytes(request),
            b'<get_report_formats report_format_id="report_format_id" details="1"/>',
        )

        request = ReportFormats.get_report_format(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b'<get_report_formats report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5" details="1"/>',
        )

    def test_get_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            ReportFormats.get_report_format(None)

        with self.assertRaises(RequiredArgument):
            ReportFormats.get_report_format("")

    def test_import_report_format(self):
        report_format_xml_string = (
            '<get_report_formats_response status="200" status_text="OK">'
            '<report_format id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
            "<name>Foobar</name>"
            "<comment>Foobar report_format</comment>"
            "<creation_time>2018-11-09T10:48:03Z</creation_time>"
            "<modification_time>2018-11-09T10:48:03Z</modification_time>"
            "</report_format>"
            "</get_report_formats_response>"
        )
        request = ReportFormats.import_report_format(report_format_xml_string)
        self.assertEqual(
            bytes(request),
            b"<create_report_format>"
            b'<get_report_formats_response status="200" status_text="OK">'
            b'<report_format id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
            b"<name>Foobar</name>"
            b"<comment>Foobar report_format</comment>"
            b"<creation_time>2018-11-09T10:48:03Z</creation_time>"
            b"<modification_time>2018-11-09T10:48:03Z</modification_time>"
            b"</report_format>"
            b"</get_report_formats_response>"
            b"</create_report_format>",
        )

    def test_import_report_format_invalid_report_format(self):
        with self.assertRaises(InvalidArgument):
            ReportFormats.import_report_format("invalid_report_format")

    def test_modify_report_format(self):
        request = ReportFormats.modify_report_format("report_format_id")
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id"/>',
        )

        request = ReportFormats.modify_report_format(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5"/>',
        )

    def test_modify_report_format_with_active(self):
        request = ReportFormats.modify_report_format(
            "report_format_id", active=True
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id">'
            b"<active>1</active>"
            b"</modify_report_format>",
        )

        request = ReportFormats.modify_report_format(
            ReportFormatType.PDF, active=False
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5">'
            b"<active>0</active>"
            b"</modify_report_format>",
        )

    def test_modify_report_format_with_name(self):
        request = ReportFormats.modify_report_format(
            "report_format_id", name="Foobar"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id">'
            b"<name>Foobar</name>"
            b"</modify_report_format>",
        )

    def test_modify_report_format_with_summary(self):
        request = ReportFormats.modify_report_format(
            "report_format_id", summary="Foobar report_format"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id">'
            b"<summary>Foobar report_format</summary>"
            b"</modify_report_format>",
        )

    def test_modify_report_format_with_param_name(self):
        request = ReportFormats.modify_report_format(
            "report_format_id", param_name="Foobar"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id">'
            b"<param>"
            b"<name>Foobar</name>"
            b"</param>"
            b"</modify_report_format>",
        )

    def test_modify_report_format_with_param_name_and_value(self):
        request = ReportFormats.modify_report_format(
            "report_format_id", param_name="Foobar", param_value="42"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_format report_format_id="report_format_id">'
            b"<param>"
            b"<name>Foobar</name>"
            b"<value>42</value>"
            b"</param>"
            b"</modify_report_format>",
        )

    def test_verify_report_format(self):
        request = ReportFormats.verify_report_format("report_format_id")
        self.assertEqual(
            bytes(request),
            b'<verify_report_format report_format_id="report_format_id"/>',
        )

        request = ReportFormats.verify_report_format(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b'<verify_report_format report_format_id="c402cc3e-b531-11e1-9163-406186ea4fc5"/>',
        )

    def test_verify_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            ReportFormats.verify_report_format(None)

        with self.assertRaises(RequiredArgument):
            ReportFormats.verify_report_format("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.policies import (
    GmpClonePolicyTestMixin,
    GmpCreatePolicyTestMixin,
    GmpDeletePolicyTestMixin,
    GmpGetPoliciesTestMixin,
    GmpGetPolicyTestMixin,
    GmpImportPolicyTestMixin,
    GmpModifyPolicySetCommentTestMixin,
    GmpModifyPolicySetFamilySelectionTestMixin,
    GmpModifyPolicySetNameTestMixin,
    GmpModifyPolicySetNvtPreferenceTestMixin,
    GmpModifyPolicySetNvtSelectionTestMixin,
    GmpModifyPolicySetScannerPreferenceTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225ClonePolicyTestCase(GmpClonePolicyTestMixin, GMPTestCase):
    pass


class Gmpv225CreatePolicyTestCase(GmpCreatePolicyTestMixin, GMPTestCase):
    pass


class Gmpv225DeletePolicyTestCase(GmpDeletePolicyTestMixin, GMPTestCase):
    pass


class Gmpv225GetPolicyTestCase(GmpGetPolicyTestMixin, GMPTestCase):
    pass


class Gmpv225GetPoliciesTestCase(GmpGetPoliciesTestMixin, GMPTestCase):
    pass


class Gmpv225ImportPolicyTestCase(GmpImportPolicyTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyPolicySetCommentTestCase(
    GmpModifyPolicySetCommentTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPolicySetFamilySelectionTestCase(
    GmpModifyPolicySetFamilySelectionTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPolicySetNvtSelectionTestCase(
    GmpModifyPolicySetNvtSelectionTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPolicySetNameTestCase(
    GmpModifyPolicySetNameTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPolicySetNvtPreferenceTestCase(
    GmpModifyPolicySetNvtPreferenceTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyPolicySetScannerPreferenceTestCase(
    GmpModifyPolicySetScannerPreferenceTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging
import socket as socketlib
import ssl

from ._connection import DEFAULT_TIMEOUT, AbstractGvmConnection

DEFAULT_GVM_PORT = 9390
DEFAULT_HOSTNAME = "127.0.0.1"
DEFAULT_KNOWN_HOSTS_FILE = ".ssh/known_hosts"

logger = logging.getLogger("gvm.connections.tls")


class TLSConnection(AbstractGvmConnection):
    """
    TLS class to connect, read and write from a remote GVM daemon via TLS
    secured socket.
    """

    def __init__(
        self,
        *,
        certfile: str | None = None,
        cafile: str | None = None,
        keyfile: str | None = None,
        hostname: str | None = DEFAULT_HOSTNAME,
        port: int | None = DEFAULT_GVM_PORT,
        password: str | None = None,
        timeout: int | float | None = DEFAULT_TIMEOUT,
    ) -> None:
        """
        Create a new TLSConnection instance.

        Args:
            timeout: Timeout in seconds for the connection.
            hostname: DNS name or IP address of the remote TLS server.
            port: Port for the TLS connection. Default is 9390.
            certfile: Path to PEM encoded certificate file. See
                `python certificates <https://docs.python.org/3/library/ssl.html#certificates>`_ for details.
            cafile: Path to PEM encoded CA file. See `python certificates <https://docs.python.org/3/library/ssl.html#certificates>`_
                for details.
            keyfile: Path to PEM encoded private key. See `python certificates <https://docs.python.org/3/library/ssl.html#certificates>`_
                for details.
            password: Password for the private key. If the password argument is not
                specified and a password is required it will be interactively prompt
                the user for a password.
        """
        super().__init__(timeout=timeout)

        self.hostname = hostname if hostname is not None else DEFAULT_HOSTNAME
        self.port = port if port is not None else DEFAULT_GVM_PORT
        self.certfile = certfile
        self.cafile = cafile
        self.keyfile = keyfile
        self.password = password

    def _new_socket(self) -> ssl.SSLSocket:
        transport_socket = socketlib.socket(
            socketlib.AF_INET, socketlib.SOCK_STREAM
        )

        if self.certfile and self.cafile and self.keyfile:
            context = ssl.create_default_context(
                ssl.Purpose.SERVER_AUTH, cafile=self.cafile
            )
            context.check_hostname = False
            context.load_cert_chain(
                certfile=self.certfile,
                keyfile=self.keyfile,
                password=self.password,
            )
            context.minimum_version = ssl.TLSVersion.TLSv1_2
        else:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
            context.minimum_version = ssl.TLSVersion.TLSv1_2
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE

        sock = context.wrap_socket(transport_socket, server_side=False)

        sock.settimeout(self._timeout)

        return sock

    def connect(self) -> None:
        self._socket = self._new_socket()
        self._socket.connect((self.hostname, int(self.port)))

    def disconnect(self):
        """Close the SSL layer then disconnect from the remote server"""
        try:
            if self._socket is not None:
                self._socket = self._socket.unwrap()
        except OSError as e:
            logger.debug("Connection closing error: %s", e)
        return super().disconnect()



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Callable
from types import TracebackType
from typing import Generic, TypeVar

from gvm.connections import GvmConnection

from .core import Connection, Request, Response

T = TypeVar("T")
Self = TypeVar("Self", bound="GvmProtocol")


def str_transform(response: Response) -> str:
    return str(response)


class GvmProtocol(Generic[T]):
    """Base class for different GVM protocols"""

    def __init__(
        self,
        connection: GvmConnection,
        *,
        transform: Callable[[Response], T] = str_transform,  # type: ignore[assignment]
    ):
        """
        Create a new GvmProtocol instance.

        Args:
            connection: Connection to use to talk with the remote daemon. See
                :mod:`gvm.connections` for possible connection types.
            transform: Optional transform callable to convert response data.
                After each request the callable gets passed the plain response data
                which can be used to check the data and/or conversion into different
                representations like a xml dom.
        """
        self._connection = connection
        self._protocol = Connection()

        self._connected = False

        self._transform_callable = transform

    def __enter__(self: Self) -> Self:
        self.connect()
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None:
        self.disconnect()

    def _read(self) -> bytes:
        """Read a command response from gvmd

        Returns:
            str: Response from server.
        """
        return self._connection.read()

    def _send(self, data: bytes) -> None:
        """Send a command to the server

        Arguments:
            data (str): Data to be send over the connection to the server
        """
        self.connect()
        self._connection.send(data)

    def is_connected(self) -> bool:
        """Status of the current connection

        Returns:
            True if a connection to the remote server has been established.
        """
        return self._connected

    def connect(self) -> None:
        """Initiates a protocol connection

        Normally connect is not called directly. Either it is called
        automatically when sending a protocol command or when using a
        `with statement <https://docs.python.org/3/reference/datamodel.html#with-statement-context-managers>`_.
        """
        if not self.is_connected():
            self._connection.connect()
            self._connected = True

    def disconnect(self) -> None:
        """Disconnect the connection

        Ends and closes the connection.
        """
        if self.is_connected():
            self._connection.disconnect()
            self._connected = False

        self._protocol.close()

    def send_command(self, cmd: str) -> str:
        """
        Send a string command to the remote daemon and return the response as
        string
        """
        return bytes(
            self._send_request(cmd.encode("utf-8", errors="ignore"))  # type: ignore[arg-type] # it seems mypy on Python < 3.11 can't handle bytes here
        ).decode("utf-8", errors="ignore")

    def _transform(self, response: Response) -> T:
        transform = self._transform_callable
        return transform(response)

    def _send_request(self, request: Request) -> Response:
        """
        Send a request to the remote daemon and return the response

        Args:
            request: The request to be send.
        """
        try:
            send_data = self._protocol.send(request)
            self._send(send_data)
            response: Response | None = None
            while not response:
                received_data = self._read()
                response = self._protocol.receive_data(received_data)
            return response
        except Exception as e:
            self.disconnect()
            raise e

    def _send_request_and_transform_response(self, request: Request) -> T:
        """
        Send a request and transform its response using the transform callable.

        Args:
            request: The request to be send.
        """
        return self._transform(self._send_request(request))



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .audits import (
    GmpCloneAuditTestMixin,
    GmpCreateAuditTestMixin,
    GmpDeleteAuditTestMixin,
    GmpGetAuditsTestMixin,
    GmpGetAuditTestMixin,
    GmpModifyAuditTestMixin,
    GmpResumeAuditTestMixin,
    GmpStartAuditTestMixin,
    GmpStopAuditTestMixin,
)


class Gmpv224CloneAuditTestCase(GmpCloneAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateAuditTestCase(GmpCreateAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteAuditTestCase(GmpDeleteAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetAuditTestCase(GmpGetAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetAuditsTestCase(GmpGetAuditsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyAuditTestCase(GmpModifyAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ResumeAuditTestCase(GmpResumeAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224StartAuditTestCase(GmpStartAuditTestMixin, Gmpv224TestCase):
    pass


class Gmpv224StopAuditTestCase(GmpStopAuditTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.help import GmpHelpTestMixin
from ...gmpv226 import GMPTestCase


class GMPHelpTestCase(GmpHelpTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetCertBundListTestMixin:
    def test_get_cpes(self):
        self.gmp.get_cert_bund_advisories()

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV"/>'
        )

    def test_get_cves_with_filter_string(self):
        self.gmp.get_cert_bund_advisories(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV" filter="foo=bar"/>'
        )

    def test_get_cves_with_filter_id(self):
        self.gmp.get_cert_bund_advisories(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV" filt_id="f1"/>'
        )

    def test_get_cves_with_name(self):
        self.gmp.get_cert_bund_advisories(name="foo")

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV" name="foo"/>'
        )

    def test_get_cves_with_details(self):
        self.gmp.get_cert_bund_advisories(details=True)

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV" details="1"/>'
        )

        self.gmp.get_cert_bund_advisories(details=False)

        self.connection.send.has_been_called_with(
            b'<get_info type="CERT_BUND_ADV" details="0"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.scan_configs import (
    GmpCloneScanConfigTestMixin,
    GmpCreateScanConfigTestMixin,
    GmpDeleteScanConfigTestMixin,
    GmpGetScanConfigsTestMixin,
    GmpGetScanConfigTestMixin,
    GmpImportScanConfigTestMixin,
    GmpModifyScanConfigSetCommentTestMixin,
    GmpModifyScanConfigSetFamilySelectionTestMixin,
    GmpModifyScanConfigSetNameTestMixin,
    GmpModifyScanConfigSetNvtPreferenceTestMixin,
    GmpModifyScanConfigSetNvtSelectionTestMixin,
    GmpModifyScanConfigSetScannerPreferenceTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPCloneScanConfigTestCase(GmpCloneScanConfigTestMixin, GMPTestCase):
    pass


class GMPCreateScanConfigTestCase(GmpCreateScanConfigTestMixin, GMPTestCase):
    pass


class GMPDeleteScanConfigTestCase(GmpDeleteScanConfigTestMixin, GMPTestCase):
    pass


class GMPGetScanConfigTestCase(GmpGetScanConfigTestMixin, GMPTestCase):
    pass


class GMPGetScanConfigsTestCase(GmpGetScanConfigsTestMixin, GMPTestCase):
    pass


class GMPImportScanConfigTestCase(GmpImportScanConfigTestMixin, GMPTestCase):
    pass


class GMPModifyScanConfigSetCommentTestCase(
    GmpModifyScanConfigSetCommentTestMixin, GMPTestCase
):
    pass


class GMPModifyScanConfigSetFamilySelectionTestCase(
    GmpModifyScanConfigSetFamilySelectionTestMixin, GMPTestCase
):
    pass


class GMPModifyScanConfigSetNvtSelectionTestCase(
    GmpModifyScanConfigSetNvtSelectionTestMixin, GMPTestCase
):
    pass


class GMPModifyScanConfigSetNameTestCase(
    GmpModifyScanConfigSetNameTestMixin, GMPTestCase
):
    pass


class GMPModifyScanConfigSetNvtPreferenceTestCase(
    GmpModifyScanConfigSetNvtPreferenceTestMixin, GMPTestCase
):
    pass


class GMPModifyScanConfigSetScannerPreferenceTestCase(
    GmpModifyScanConfigSetScannerPreferenceTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument


class GmpDeleteAgentsTestMixin:
    def test_delete_agents(self):
        self.gmp.delete_agents(agent_ids=["agent-123", "agent-456"])

        self.connection.send.has_been_called_with(
            b"<delete_agent>"
            b'<agents><agent id="agent-123"/><agent id="agent-456"/></agents>'
            b"</delete_agent>"
        )

    def test_delete_agents_without_ids(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.delete_agents(agent_ids=[])



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
Package for supported Greenbone HTTP APIs.

Currently only `openvasd version 1 <https://greenbone.github.io/scanner-api/#/>`_ is supported.
"""



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateHostTestMixin:
    def test_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_host(name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_host(name="")

    def test_create_host_asset(self):
        self.gmp.create_host(name="ipsum")

        self.connection.send.has_been_called_with(
            b"<create_asset>"
            b"<asset>"
            b"<type>host</type>"
            b"<name>ipsum</name>"
            b"</asset>"
            b"</create_asset>"
        )

    def test_create_asset_with_comment(self):
        self.gmp.create_host(name="ipsum", comment="lorem")

        self.connection.send.has_been_called_with(
            b"<create_asset>"
            b"<asset>"
            b"<type>host</type>"
            b"<name>ipsum</name>"
            b"<comment>lorem</comment>"
            b"</asset>"
            b"</create_asset>"
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import check_port, to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._severity import Severity


class Notes:
    @classmethod
    def create_note(
        cls,
        text: str,
        nvt_oid: str,
        *,
        days_active: int | None = None,
        hosts: list[str] | None = None,
        port: str | None = None,
        result_id: EntityID | None = None,
        severity: Severity | None = None,
        task_id: EntityID | None = None,
    ) -> Request:
        """Create a new note

        Args:
            text: Text of the new note
            nvt_oid: OID of the nvt to which note applies
            days_active: Days note will be active. -1 on
                always, 0 off
            hosts: A list of host addresses
            port: Port to which the override applies, needs to be a string
                  in the form {number}/{protocol}
            result_id: UUID of a result to which note applies
            severity: Severity to which note applies
            task_id: UUID of task to which note applies
        """
        if not text:
            raise RequiredArgument(
                function=cls.create_note.__name__, argument="text"
            )

        if not nvt_oid:
            raise RequiredArgument(
                function=cls.create_note.__name__, argument="nvt_oid"
            )

        cmd = XmlCommand("create_note")
        cmd.add_element("text", text)
        cmd.add_element("nvt", attrs={"oid": nvt_oid})

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", to_comma_list(hosts))

        if port:
            if check_port(port):
                cmd.add_element("port", port)
            else:
                raise InvalidArgument(
                    function=cls.create_note.__name__, argument="port"
                )

        if result_id:
            cmd.add_element("result", attrs={"id": str(result_id)})

        if severity is not None:
            cmd.add_element("severity", str(severity))

        if task_id:
            cmd.add_element("task", attrs={"id": str(task_id)})

        return cmd

    @classmethod
    def modify_note(
        cls,
        note_id: EntityID,
        text: str,
        *,
        days_active: int | None = None,
        hosts: list[str] | None = None,
        port: str | None = None,
        result_id: EntityID | None = None,
        severity: Severity | None = None,
        task_id: EntityID | None = None,
    ) -> Request:
        """Modify a note

        Args:
            note_id: The UUID of the note to modify
            text: Text of the note
            days_active: Days note will be active. -1 on always, 0 off
            hosts: A list of host addresses
            port: Port to which the override applies, needs to be a string
                  in the form {number}/{protocol}
            result_id: UUID of a result to which note applies
            severity: Severity to which note applies
            task_id: UUID of task to which note applies
        """
        if not note_id:
            raise RequiredArgument(
                function=cls.modify_note.__name__, argument="note_id"
            )

        if not text:
            raise RequiredArgument(
                function=cls.modify_note.__name__, argument="text"
            )

        cmd = XmlCommand("modify_note")
        cmd.set_attribute("note_id", str(note_id))
        cmd.add_element("text", text)

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", to_comma_list(hosts))

        if port:
            if check_port(port):
                cmd.add_element("port", port)
            else:
                raise InvalidArgument(
                    function=cls.modify_note.__name__, argument="port"
                )

        if result_id:
            cmd.add_element("result", attrs={"id": str(result_id)})

        if severity is not None:
            cmd.add_element("severity", str(severity))

        if task_id:
            cmd.add_element("task", attrs={"id": str(task_id)})

        return cmd

    @classmethod
    def clone_note(cls, note_id: EntityID) -> Request:
        """Clone an existing note

        Args:
            note_id: UUID of an existing note to clone from
        """
        if not note_id:
            raise RequiredArgument(
                function=cls.clone_note.__name__, argument="note_id"
            )

        cmd = XmlCommand("create_note")
        cmd.add_element("copy", str(note_id))
        return cmd

    @classmethod
    def delete_note(
        cls, note_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing note

        Args:
            note_id: UUID of the note to be deleted.
            ultimate: Whether to remove entirely or to the trashcan.
        """
        if not note_id:
            raise RequiredArgument(
                function=cls.delete_note.__name__, argument="note_id"
            )

        cmd = XmlCommand("delete_note")
        cmd.set_attribute("note_id", str(note_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @classmethod
    def get_notes(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
        result: bool | None = None,
    ) -> Request:
        """Request a list of notes

        Args:
            filter_string: Filter notes by a string
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: Add info about connected results and tasks
            result: Return the details of possible connected results.
        """
        cmd = XmlCommand("get_notes")
        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if result is not None:
            cmd.set_attribute("result", to_bool(result))

        return cmd

    @classmethod
    def get_note(cls, note_id: EntityID) -> Request:
        """Request a single note

        Arguments:
            note_id: UUID of an existing note
        """
        if not note_id:
            raise RequiredArgument(
                function=cls.get_note.__name__, argument="note_id"
            )

        cmd = XmlCommand("get_notes")
        cmd.set_attribute("note_id", str(note_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.hosts import (
    GmpCreateHostTestMixin,
    GmpDeleteHostTestMixin,
    GmpGetHostsTestMixin,
    GmpGetHostTestMixin,
    GmpModifyHostTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CreateHostTestCase(GmpCreateHostTestMixin, GMPTestCase):
    pass


class Gmpv225DeleteHostTestCase(GmpDeleteHostTestMixin, GMPTestCase):
    pass


class Gmpv225GetHostTestCase(GmpGetHostTestMixin, GMPTestCase):
    pass


class Gmpv225GetHostsTestCase(GmpGetHostsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyHostTestCase(GmpModifyHostTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Policies

POLICY_XML_STRING = (
    '<get_configs_response status="200" status_text="OK">'
    '<config id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
    "<name>Foobar</name>"
    "<comment>Foobar config</comment>"
    "<creation_time>2018-11-09T10:48:03Z</creation_time>"
    "<modification_time>2018-11-09T10:48:03Z</modification_time>"
    "</config>"
    "</get_configs_response>"
)


class PoliciesTestCase(unittest.TestCase):
    def test_clone_policy(self):
        request = Policies.clone_policy("policy_id")
        self.assertEqual(
            bytes(request),
            b"<create_config><copy>policy_id</copy></create_config>",
        )

    def test_clone_policy_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.clone_policy(None)

        with self.assertRaises(RequiredArgument):
            Policies.clone_policy("")

    def test_create_policy(self):
        request = Policies.create_policy("name")
        self.assertEqual(
            bytes(request),
            b"<create_config>"
            b"<copy>085569ce-73ed-11df-83c3-002264764cea</copy>"
            b"<name>name</name>"
            b"<usage_type>policy</usage_type>"
            b"</create_config>",
        )

    def test_create_policy_with_policy_id(self):
        request = Policies.create_policy("name", policy_id="policy_id")
        self.assertEqual(
            bytes(request),
            b"<create_config>"
            b"<copy>policy_id</copy>"
            b"<name>name</name>"
            b"<usage_type>policy</usage_type>"
            b"</create_config>",
        )

    def test_create_policy_with_comment(self):
        request = Policies.create_policy("name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_config>"
            b"<comment>comment</comment>"
            b"<copy>085569ce-73ed-11df-83c3-002264764cea</copy>"
            b"<name>name</name>"
            b"<usage_type>policy</usage_type>"
            b"</create_config>",
        )

    def test_delete_policy(self):
        request = Policies.delete_policy("policy_id")
        self.assertEqual(
            bytes(request),
            b'<delete_config config_id="policy_id" ultimate="0"/>',
        )

    def test_delete_policy_with_ultimate(self):
        request = Policies.delete_policy("policy_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_config config_id="policy_id" ultimate="1"/>',
        )

        request = Policies.delete_policy("policy_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_config config_id="policy_id" ultimate="0"/>',
        )

    def test_get_policies(self):
        request = Policies.get_policies()
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy"/>',
        )

    def test_get_policies_with_trash(self):
        request = Policies.get_policies(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" trash="1"/>',
        )

        request = Policies.get_policies(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" trash="0"/>',
        )

    def test_get_policies_with_details(self):
        request = Policies.get_policies(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" details="1"/>',
        )

        request = Policies.get_policies(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" details="0"/>',
        )

    def test_get_policies_with_families(self):
        request = Policies.get_policies(families=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" families="1"/>',
        )

        request = Policies.get_policies(families=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" families="0"/>',
        )

    def test_get_policies_with_preferences(self):
        request = Policies.get_policies(preferences=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" preferences="1"/>',
        )

        request = Policies.get_policies(preferences=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" preferences="0"/>',
        )

    def test_get_policies_with_audits(self):
        request = Policies.get_policies(audits=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" tasks="1"/>',
        )

        request = Policies.get_policies(audits=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="policy" tasks="0"/>',
        )

    def test_get_policy(self):
        request = Policies.get_policy("policy_id")
        self.assertEqual(
            bytes(request),
            b'<get_configs config_id="policy_id" usage_type="policy" details="1"/>',
        )

    def test_get_policy_with_audits(self):
        request = Policies.get_policy("policy_id", audits=True)
        self.assertEqual(
            bytes(request),
            b'<get_configs config_id="policy_id" usage_type="policy" tasks="1" details="1"/>',
        )

        request = Policies.get_policy("policy_id", audits=False)
        self.assertEqual(
            bytes(request),
            b'<get_configs config_id="policy_id" usage_type="policy" tasks="0" details="1"/>',
        )

    def test_get_policy_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.get_policy(None)

        with self.assertRaises(RequiredArgument):
            Policies.get_policy("")

    def test_import_policy(self):
        request = Policies.import_policy(POLICY_XML_STRING)
        self.assertEqual(
            bytes(request),
            b"<create_config>"
            b'<get_configs_response status="200" status_text="OK">'
            b'<config id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
            b"<name>Foobar</name>"
            b"<comment>Foobar config</comment>"
            b"<creation_time>2018-11-09T10:48:03Z</creation_time>"
            b"<modification_time>2018-11-09T10:48:03Z</modification_time>"
            b"</config>"
            b"</get_configs_response>"
            b"</create_config>",
        )

    def test_import_policy_missing_policy_xml(self):
        with self.assertRaises(RequiredArgument):
            Policies.import_policy(None)

        with self.assertRaises(RequiredArgument):
            Policies.import_policy("")

    def test_import_policy_invalid_xml(self):
        with self.assertRaises(InvalidArgument):
            Policies.import_policy("abcdef")

    def test_modify_policy_set_nvt_preference(self):
        request = Policies.modify_policy_set_nvt_preference(
            "policy_id", "name", "oid"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<preference>"
            b'<nvt oid="oid"/>'
            b"<name>name</name>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_policy_set_nvt_preference_with_value(self):
        request = Policies.modify_policy_set_nvt_preference(
            "policy_id", "name", "oid", value="bar"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<preference>"
            b'<nvt oid="oid"/>'
            b"<name>name</name>"
            b"<value>YmFy</value>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_policy_set_nvt_preference_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference(None, "name", "oid")

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference("", "name", "oid")

    def test_modify_policy_set_nvt_preference_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference("policy_id", None, "oid")

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference("policy_id", "", "oid")

    def test_modify_policy_set_nvt_preference_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference("policy_id", "name", None)

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_preference("policy_id", "name", "")

    def test_modify_policy_set_name(self):
        request = Policies.modify_policy_set_name("policy_id", "name")
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<name>name</name>"
            b"</modify_config>",
        )

    def test_modify_policy_set_name_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_name(None, "name")

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_name("", "name")

    def test_modify_policy_set_name_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_name("policy_id", None)

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_name("policy_id", "")

    def test_modify_policy_set_comment(self):
        request = Policies.modify_policy_set_comment("policy_id", None)
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<comment></comment>"
            b"</modify_config>",
        )

        request = Policies.modify_policy_set_comment("policy_id", "comment")
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<comment>comment</comment>"
            b"</modify_config>",
        )

    def test_modify_policy_set_comment_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_comment(None, "comment")

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_comment("", "comment")

    def test_modify_policy_set_scanner_preference(self):
        request = Policies.modify_policy_set_scanner_preference(
            "policy_id",
            "name",
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<preference>"
            b"<name>name</name>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_policy_set_scanner_preference_with_value(self):
        request = Policies.modify_policy_set_scanner_preference(
            "policy_id",
            "name",
            value="bar",
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<preference>"
            b"<name>name</name>"
            b"<value>YmFy</value>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_policy_set_scanner_preference_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_scanner_preference(None, "name")

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_scanner_preference("", "name")

    def test_modify_policy_set_scanner_preference_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_scanner_preference("policy_id", None)

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_scanner_preference("policy_id", "")

    def test_modify_policy_set_nvt_selection(self):
        request = Policies.modify_policy_set_nvt_selection(
            "policy_id",
            "name",
            ["oid"],
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<nvt_selection>"
            b"<family>name</family>"
            b'<nvt oid="oid"/>'
            b"</nvt_selection>"
            b"</modify_config>",
        )

        request = Policies.modify_policy_set_nvt_selection(
            "policy_id",
            "name",
            ["oid1", "oid2"],
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<nvt_selection>"
            b"<family>name</family>"
            b'<nvt oid="oid1"/>'
            b'<nvt oid="oid2"/>'
            b"</nvt_selection>"
            b"</modify_config>",
        )

    def test_modify_policy_set_nvt_selection_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_selection(None, "name", ["oid"])

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_selection("", "name", ["oid"])

    def test_modify_policy_set_nvt_selection_missing_family(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_selection("policy_id", None, ["oid"])

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_nvt_selection("policy_id", "", ["oid"])

    def test_modify_policy_set_family_selection(self):
        request = Policies.modify_policy_set_family_selection(
            "policy_id",
            [("name", True, True)],
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>name</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = Policies.modify_policy_set_family_selection(
            "policy_id",
            [("name1", True, True), ("name2", True, False)],
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>name1</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>name2</name>"
            b"<all>0</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

    def test_modify_policy_set_family_selection_with_auto_add_new_families(
        self,
    ):
        request = Policies.modify_policy_set_family_selection(
            "policy_id",
            [("name", True, True)],
            auto_add_new_families=True,
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>name</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = Policies.modify_policy_set_family_selection(
            "policy_id",
            [("name", True, True)],
            auto_add_new_families=False,
        )
        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="policy_id">'
            b"<family_selection>"
            b"<growing>0</growing>"
            b"<family>"
            b"<name>name</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

    def test_modify_policy_set_family_selection_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_family_selection(
                None, [("name", True, True)]
            )

        with self.assertRaises(RequiredArgument):
            Policies.modify_policy_set_family_selection(
                "", [("name", True, True)]
            )

    def test_modify_policy_set_family_selection_invalid_families(self):
        with self.assertRaises(InvalidArgument):
            Policies.modify_policy_set_family_selection(
                "policy_id", [("name", True, True, True)]
            )

        with self.assertRaises(InvalidArgument):
            Policies.modify_policy_set_family_selection(
                "policy_id", [("name", True)]
            )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import (
    AlertCondition,
    AlertEvent,
    AlertMethod,
    Alerts,
    ReportFormatType,
)


class AlertsTestCase(unittest.TestCase):
    def test_create_alerts(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always</condition>"
            b"<event>Task run status changed</event>"
            b"<method>Email</method>"
            b"</create_alert>",
        )

    def test_create_alert_with_filter_id(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            filter_id="f1",
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always</condition>"
            b"<event>Task run status changed</event>"
            b"<method>Email</method>"
            b'<filter id="f1"/>'
            b"</create_alert>",
        )

    def test_create_alert_with_comment(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            comment="bar",
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always</condition>"
            b"<event>Task run status changed</event>"
            b"<method>Email</method>"
            b"<comment>bar</comment>"
            b"</create_alert>",
        )

    def test_create_alert_with_condition_data(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            condition_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always<data>bar<name>foo</name></data></condition>"
            b"<event>Task run status changed</event>"
            b"<method>Email</method>"
            b"</create_alert>",
        )

    def test_create_alert_with_event_data(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            event_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always</condition>"
            b"<event>Task run status changed"
            b"<data>bar<name>foo</name></data>"
            b"</event>"
            b"<method>Email</method>"
            b"</create_alert>",
        )

    def test_create_alert_with_method_data(self):
        request = Alerts.create_alert(
            name="foo",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            method_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b"<create_alert>"
            b"<name>foo</name>"
            b"<condition>Always</condition>"
            b"<event>Task run status changed</event>"
            b"<method>Email<data>bar<name>foo</name></data></method>"
            b"</create_alert>",
        )

    def test_create_alert_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                None,
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_missing_event(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event=None,
                method=AlertMethod.EMAIL,
            )

        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event="",
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_missing_condition(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition=None,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition="",
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_missing_method(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=None,
            )

        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method="",
            )

    def test_create_alert_invalid_condition(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                "foo",
                condition="foo",
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_invalid_event(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event="foo",
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_invalid_method(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                "foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method="foo",
            )

    def test_create_alert_invalid_condition_for_secinfo(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.SEVERITY_AT_LEAST,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_invalid_method_for_secinfo(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.HTTP_GET,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.ALEMBA_VFIRE,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.SOURCEFIRE_CONNECTOR,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.START_TASK,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.TIPPINGPOINT_SMS,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.VERINICE_CONNECTOR,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.HTTP_GET,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.ALEMBA_VFIRE,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.SOURCEFIRE_CONNECTOR,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.START_TASK,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.TIPPINGPOINT_SMS,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.VERINICE_CONNECTOR,
            )

    def test_create_alert_missing_method_for_ticket_received(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=None,
            )

    def test_create_alert_missing_condition_for_ticket_received(self):
        with self.assertRaises(RequiredArgument):
            Alerts.create_alert(
                name="foo",
                condition=None,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_invalid_method_for_ticket_received(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.HTTP_GET,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.SCP,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.SEND,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.SMB,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.SNMP,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.ALEMBA_VFIRE,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.VERINICE_CONNECTOR,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.TIPPINGPOINT_SMS,
            )

        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.SOURCEFIRE_CONNECTOR,
            )

    def test_create_alert_invalid_condition_for_task_run_status_changed(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.ERROR,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.EMAIL,
            )

    def test_create_alert_invalid_condition_for_ticket_received(self):
        with self.assertRaises(InvalidArgument):
            Alerts.create_alert(
                name="foo",
                condition=AlertCondition.FILTER_COUNT_AT_LEAST,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.EMAIL,
            )

    def test_modify_alert(self):
        request = Alerts.modify_alert(
            alert_id="a1",
        )

        self.assertEqual(bytes(request), b'<modify_alert alert_id="a1"/>')

    def test_modify_alert_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.modify_alert(alert_id="")

        with self.assertRaises(RequiredArgument):
            Alerts.modify_alert(alert_id=None)

    def test_modify_alert_with_comment(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            comment="foo",
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1"><comment>foo</comment></modify_alert>',
        )

    def test_modify_alert_with_name(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            name="foo",
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1"><name>foo</name></modify_alert>',
        )

    def test_modify_alert_with_filter_id(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            filter_id="f1",
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1"><filter id="f1"/></modify_alert>',
        )

    def test_modify_alert_with_event_data(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            event_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1">'
            b"<condition>Always</condition>"
            b"<method>Email</method>"
            b"<event>Task run status changed"
            b"<data>bar<name>foo</name></data>"
            b"</event>"
            b"</modify_alert>",
        )

    def test_modify_alert_with_condition_data(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            condition_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1">'
            b"<condition>Always<data>bar<name>foo</name></data></condition>"
            b"<method>Email</method>"
            b"<event>Task run status changed</event>"
            b"</modify_alert>",
        )

    def test_modify_alert_with_method_data(self):
        request = Alerts.modify_alert(
            alert_id="a1",
            condition=AlertCondition.ALWAYS,
            event=AlertEvent.TASK_RUN_STATUS_CHANGED,
            method=AlertMethod.EMAIL,
            method_data={"foo": "bar"},
        )

        self.assertEqual(
            bytes(request),
            b'<modify_alert alert_id="a1">'
            b"<condition>Always</condition>"
            b"<method>Email<data>bar<name>foo</name></data></method>"
            b"<event>Task run status changed</event>"
            b"</modify_alert>",
        )

    def test_modify_alert_invalid_condition(self):
        with self.assertRaises(InvalidArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition="bar",
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.SCP,
            )

    def test_modify_alert_invalid_event(self):
        with self.assertRaises(InvalidArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition=AlertCondition.ALWAYS,
                event="lorem",
                method=AlertMethod.SCP,
            )

    def test_modify_alert_invalid_method(self):
        with self.assertRaises(InvalidArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method="ipsum",
            )

    def test_modify_alert_with_event_missing_method(self):
        with self.assertRaisesRegex(RequiredArgument, "method is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                condition=AlertCondition.ALWAYS,
            )

        with self.assertRaisesRegex(RequiredArgument, "method is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                condition=AlertCondition.ALWAYS,
            )

        with self.assertRaisesRegex(RequiredArgument, "method is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                condition=AlertCondition.ALWAYS,
            )

    def test_modify_alert_with_event_missing_condition(self):
        with self.assertRaisesRegex(RequiredArgument, "condition is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.TASK_RUN_STATUS_CHANGED,
                method=AlertMethod.SCP,
            )

        with self.assertRaisesRegex(RequiredArgument, "condition is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.NEW_SECINFO_ARRIVED,
                method=AlertMethod.SCP,
            )

        with self.assertRaisesRegex(RequiredArgument, "condition is required"):
            Alerts.modify_alert(
                alert_id="a1",
                event=AlertEvent.UPDATED_SECINFO_ARRIVED,
                method=AlertMethod.SCP,
            )

    def test_modify_alert_invalid_condition_for_secinfo(self):
        with self.assertRaises(InvalidArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition="Severity at least",
                event="Updated SecInfo arrived",
                method="Email",
            )

    def test_modify_alert_invalid_method_for_secinfo(self):
        with self.assertRaises(InvalidArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition="Always",
                event="Updated SecInfo arrived",
                method="HTTP Get",
            )

    def test_modify_missing_method_for_ticket_received(self):
        with self.assertRaises(RequiredArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition=AlertCondition.ALWAYS,
                event=AlertEvent.TICKET_RECEIVED,
                method=None,
            )

    def test_modify_missing_condition_for_ticket_received(self):
        with self.assertRaises(RequiredArgument):
            Alerts.modify_alert(
                alert_id="a1",
                condition=None,
                event=AlertEvent.TICKET_RECEIVED,
                method=AlertMethod.EMAIL,
            )

    def test_clone_alert(self):
        request = Alerts.clone_alert(alert_id="a1")

        self.assertEqual(
            bytes(request), b"<create_alert><copy>a1</copy></create_alert>"
        )

    def test_clone_alert_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.clone_alert(alert_id="")

        with self.assertRaises(RequiredArgument):
            Alerts.clone_alert(alert_id=None)

    def test_delete_alert(self):
        request = Alerts.delete_alert(alert_id="a1")

        self.assertEqual(
            bytes(request), b'<delete_alert alert_id="a1" ultimate="0"/>'
        )

    def test_delete_alert_ultimate(self):
        request = Alerts.delete_alert(alert_id="a1", ultimate=True)

        self.assertEqual(
            bytes(request), b'<delete_alert alert_id="a1" ultimate="1"/>'
        )

        request = Alerts.delete_alert(alert_id="a1", ultimate=False)

        self.assertEqual(
            bytes(request), b'<delete_alert alert_id="a1" ultimate="0"/>'
        )

    def test_delete_alert_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.delete_alert(alert_id="")

        with self.assertRaises(RequiredArgument):
            Alerts.delete_alert(alert_id=None)

    def test_test_alert(self):
        request = Alerts.test_alert(alert_id="a1")

        self.assertEqual(bytes(request), b'<test_alert alert_id="a1"/>')

    def test_test_alert_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.test_alert(alert_id="")

        with self.assertRaises(RequiredArgument):
            Alerts.test_alert(alert_id=None)

    def test_trigger_alert(self):
        request = Alerts.trigger_alert(alert_id="a1", report_id="r1")

        self.assertEqual(
            bytes(request), b'<get_reports report_id="r1" alert_id="a1"/>'
        )

    def test_trigger_alert_with_filter_string(self):
        request = Alerts.trigger_alert(
            alert_id="a1", report_id="r1", filter_string="name=foo"
        )

        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="r1" alert_id="a1" filter="name=foo"/>',
        )

    def test_trigger_alert_with_filter_id(self):
        request = Alerts.trigger_alert(
            alert_id="a1", report_id="r1", filter_id="f1"
        )

        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="r1" alert_id="a1" filt_id="f1"/>',
        )

    def test_trigger_alert_with_report_format_id(self):
        request = Alerts.trigger_alert(
            alert_id="a1", report_id="r1", report_format_id="bar"
        )

        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="r1" alert_id="a1" format_id="bar"/>',
        )

    def test_trigger_alert_with_report_format_type(self):
        request = Alerts.trigger_alert(
            alert_id="a1",
            report_id="r1",
            report_format_id=ReportFormatType.SVG,
        )

        self.assertEqual(
            bytes(request),
            '<get_reports report_id="r1" alert_id="a1" '
            f'format_id="{ReportFormatType.SVG}"/>'.encode(),
        )

    def test_trigger_alert_with_delta_report_id(self):
        request = Alerts.trigger_alert(
            alert_id="a1", report_id="r1", delta_report_id="d1"
        )

        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="r1" alert_id="a1" delta_report_id="d1"/>',
        )

    def test_trigger_alert_missing_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.trigger_alert(alert_id="", report_id="r1")

        with self.assertRaises(RequiredArgument):
            Alerts.trigger_alert(alert_id=None, report_id="r1")

    def test_trigger_alert_missing_report_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.trigger_alert(alert_id="a1", report_id="")

        with self.assertRaises(RequiredArgument):
            Alerts.trigger_alert(alert_id="a1", report_id=None)

    def test_get_alerts(self):
        request = Alerts.get_alerts()

        self.assertEqual(bytes(request), b"<get_alerts/>")

    def test_get_alerts_with_trash(self):
        request = Alerts.get_alerts(trash=True)

        self.assertEqual(bytes(request), b'<get_alerts trash="1"/>')

        request = Alerts.get_alerts(trash=False)

        self.assertEqual(bytes(request), b'<get_alerts trash="0"/>')

    def test_get_alerts_with_filter_string(self):
        request = Alerts.get_alerts(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_alerts filter="foo=bar"/>')

    def test_get_alerts_with_filter_id(self):
        request = Alerts.get_alerts(filter_id="f1")

        self.assertEqual(bytes(request), b'<get_alerts filt_id="f1"/>')

    def test_get_alerts_with_tasks(self):
        request = Alerts.get_alerts(tasks=True)

        self.assertEqual(bytes(request), b'<get_alerts tasks="1"/>')

        request = Alerts.get_alerts(tasks=False)

        self.assertEqual(bytes(request), b'<get_alerts tasks="0"/>')

    def test_get_alert(self):
        request = Alerts.get_alert(alert_id="a1")

        self.assertEqual(bytes(request), b'<get_alerts alert_id="a1"/>')

    def test_get_alert_with_tasks(self):
        request = Alerts.get_alert(alert_id="a1", tasks=True)

        self.assertEqual(
            bytes(request), b'<get_alerts alert_id="a1" tasks="1"/>'
        )

        request = Alerts.get_alert(alert_id="a1", tasks=False)

        self.assertEqual(
            bytes(request), b'<get_alerts alert_id="a1" tasks="0"/>'
        )

    def test_get_alert_invalid_alert_id(self):
        with self.assertRaises(RequiredArgument):
            Alerts.get_alert(alert_id=None)

        with self.assertRaises(RequiredArgument):
            Alerts.get_alert(alert_id="")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import ReportFormatType


class GmpCloneReportFormatTestMixin:
    def test_clone(self):
        self.gmp.clone_report_format("a1")

        self.connection.send.has_been_called_with(
            b"<create_report_format><copy>a1</copy></create_report_format>"
        )

    def test_missing_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.clone_report_format("")

        with self.assertRaises(RequiredArgument):
            self.gmp.clone_report_format(None)

    def test_clone_with_type(self):
        self.gmp.clone_report_format(ReportFormatType.SVG)

        report_format_id = ReportFormatType.from_string("svg").value

        self.connection.send.has_been_called_with(
            "<create_report_format>"
            f"<copy>{report_format_id}</copy>"
            "</create_report_format>".encode()
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument
from gvm.protocols.gmp._gmp226 import ReportConfigParameter


class GMPCreateReportConfigTestMixin:
    def test_create_report_config(self):
        self.gmp.create_report_config("r1", "report_format_id")

        self.connection.send.has_been_called_with(
            b"<create_report_config>"
            b"<name>r1</name>"
            b'<report_format id="report_format_id"/>'
            b"</create_report_config>"
        )

    def test_create_report_config_with_comment(self):
        self.gmp.create_report_config("r1", "report_format_id", comment="foo")

        self.connection.send.has_been_called_with(
            b"<create_report_config>"
            b"<name>r1</name>"
            b'<report_format id="report_format_id"/>'
            b"<comment>foo</comment>"
            b"</create_report_config>"
        )

    def test_create_report_config_with_params(self):
        self.gmp.create_report_config(
            "r1",
            "report_format_id",
            params=[
                ReportConfigParameter("name", "value"),
                ReportConfigParameter("name2", "value2", use_default=True),
                ReportConfigParameter("name3", use_default=True),
            ],
        )

        self.connection.send.has_been_called_with(
            b"<create_report_config>"
            b"<name>r1</name>"
            b'<report_format id="report_format_id"/>'
            b'<param><name>name</name><value use_default="0">value</value></param>'
            b'<param><name>name2</name><value use_default="1"/></param>'
            b'<param><name>name3</name><value use_default="1"/></param>'
            b"</create_report_config>",
        )

    def test_create_report_config_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_report_config(None, "report_format_id")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_report_config("", "report_format_id")

    def test_create_report_config_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_report_config("r1", None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_report_config("r1", "")



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from .agent_groups.test_clone_agent_group import (
    GmpCloneAgentGroupTestMixin,
)
from .agent_groups.test_create_agent_group import (
    GmpCreateAgentGroupTestMixin,
)
from .agent_groups.test_delete_agent_group import (
    GmpDeleteAgentGroupTestMixin,
)
from .agent_groups.test_get_agent_group import (
    GmpGetAgentGroupTestMixin,
)
from .agent_groups.test_get_agent_groups import (
    GmpGetAgentGroupsTestMixin,
)
from .agent_groups.test_modify_agent_group import (
    GmpModifyAgentGroupTestMixin,
)


class GMPGetAgentGroupsTestCase(GmpGetAgentGroupsTestMixin, GMPTestCase):
    pass


class GMPGetAgentGroupTestCase(GmpGetAgentGroupTestMixin, GMPTestCase):
    pass


class GMPCreateAgentGroupTestCase(GmpCreateAgentGroupTestMixin, GMPTestCase):
    pass


class GMPCloneAgentGroupTestCase(GmpCloneAgentGroupTestMixin, GMPTestCase):
    pass


class GMPModifyAgentGroupTestCase(GmpModifyAgentGroupTestMixin, GMPTestCase):
    pass


class GMPDeleteAgentGroupTestCase(GmpDeleteAgentGroupTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_port_list import GmpClonePortListTestMixin
from .test_create_port_list import GmpCreatePortListTestMixin
from .test_create_port_range import GmpCreatePortRangeTestMixin
from .test_delete_port_list import GmpDeletePortListTestMixin
from .test_delete_port_range import GmpDeletePortRangeTestMixin
from .test_get_port_list import GmpGetPortListTestMixin
from .test_get_port_lists import GmpGetPortListsTestMixin
from .test_modify_port_list import GmpModifyPortListTestMixin

__all__ = (
    "GmpClonePortListTestMixin",
    "GmpCreatePortListTestMixin",
    "GmpCreatePortRangeTestMixin",
    "GmpDeletePortListTestMixin",
    "GmpDeletePortRangeTestMixin",
    "GmpGetPortListTestMixin",
    "GmpGetPortListsTestMixin",
    "GmpModifyPortListTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Optional

from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class FilterType(Enum):
    """Enum for filter types"""

    ALERT = "alert"
    ASSET = "asset"
    SCAN_CONFIG = "config"
    CREDENTIAL = "credential"
    FILTER = "filter"
    GROUP = "group"
    HOST = "host"
    NOTE = "note"
    OPERATING_SYSTEM = "os"
    OVERRIDE = "override"
    PERMISSION = "permission"
    PORT_LIST = "port_list"
    REPORT = "report"
    REPORT_FORMAT = "report_format"
    RESULT = "result"
    ROLE = "role"
    SCHEDULE = "schedule"
    ALL_SECINFO = "secinfo"
    TAG = "tag"
    TARGET = "target"
    TASK = "task"
    TICKET = "ticket"
    TLS_CERTIFICATE = "tls_certificate"
    USER = "user"
    VULNERABILITY = "vuln"

    @classmethod
    def from_string(
        cls,
        filter_type: str | None,
    ) -> Optional["FilterType"]:
        """Convert a filter type string to an actual FilterType instance

        Args:
            filter_type (str): Filter type string to convert to a FilterType
        """
        if filter_type == "vuln":
            return cls.VULNERABILITY

        if filter_type == "os":
            return cls.OPERATING_SYSTEM

        if filter_type == "config":
            return cls.SCAN_CONFIG

        if filter_type == "secinfo":
            return cls.ALL_SECINFO

        return super().from_string(filter_type)


class Filters:
    @classmethod
    def clone_filter(cls, filter_id: EntityID) -> Request:
        """Clone a filter

        Args:
            filter_id: ID of the filter to clone
        """
        if not filter_id:
            raise RequiredArgument(
                function=cls.clone_filter.__name__, argument="filter_id"
            )

        cmd = XmlCommand("create_filter")
        cmd.add_element("copy", str(filter_id))
        return cmd

    @classmethod
    def create_filter(
        cls,
        name: str,
        *,
        filter_type: FilterType | None = None,
        comment: str | None = None,
        term: str | None = None,
    ) -> Request:
        """Create a new filter

        Args:
            name: Name of the new filter
            filter_type: Filter for entity type
            comment: Comment for the filter
            term: Filter term e.g. 'name=foo'
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_filter.__name__, argument="name"
            )

        cmd = XmlCommand("create_filter")
        cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if term:
            cmd.add_element("term", term)

        if filter_type:
            if not isinstance(filter_type, FilterType):
                filter_type = FilterType(filter_type)

            cmd.add_element("type", filter_type.value)

        return cmd

    @classmethod
    def delete_filter(
        cls, filter_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing filter

        Args:
            filter_id: UUID of the filter to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not filter_id:
            raise RequiredArgument(
                function=cls.delete_filter.__name__, argument="filter_id"
            )

        cmd = XmlCommand("delete_filter")
        cmd.set_attribute("filter_id", str(filter_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_filters(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        alerts: bool | None = None,
    ) -> Request:
        """Request a list of filters

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan filters instead
            alerts: Whether to include list of alerts that use the filter.
        """
        cmd = XmlCommand("get_filters")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if alerts is not None:
            cmd.set_attribute("alerts", to_bool(alerts))

        return cmd

    @classmethod
    def get_filter(
        cls, filter_id: EntityID, *, alerts: bool | None = None
    ) -> Request:
        """Request a single filter

        Args:
            filter_id: UUID of an existing filter
            alerts: Whether to include list of alerts that use the filter.
        """
        cmd = XmlCommand("get_filters")

        if not filter_id:
            raise RequiredArgument(
                function=cls.get_filter.__name__, argument="filter_id"
            )

        cmd.set_attribute("filter_id", str(filter_id))

        if alerts is not None:
            cmd.set_attribute("alerts", to_bool(alerts))

        return cmd

    @classmethod
    def modify_filter(
        cls,
        filter_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
        term: str | None = None,
        filter_type: FilterType | None = None,
    ) -> Request:
        """Modifies an existing filter.

        Args:
            filter_id: UUID of the filter to be modified
            comment: Comment on filter.
            name: Name of filter.
            term: Filter term.
            filter_type: Resource type filter applies to.
        """
        if not filter_id:
            raise RequiredArgument(
                function=cls.modify_filter.__name__, argument="filter_id"
            )

        cmd = XmlCommand("modify_filter")
        cmd.set_attribute("filter_id", str(filter_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if term:
            cmd.add_element("term", term)

        if filter_type:
            if not isinstance(filter_type, FilterType):
                filter_type = FilterType(filter_type)

            cmd.add_element("type", filter_type.value)

        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from enum import Enum as PythonEnum
from typing import Any, TypeVar

from gvm.errors import InvalidArgument

Self = TypeVar("Self", bound="Enum")


class Enum(PythonEnum):
    """
    Base class for Enums in python-gvm
    """

    @classmethod
    def _missing_(cls: type[Self], value: Any) -> Self | None:
        if isinstance(value, PythonEnum):
            return cls.from_string(value.name)
        return cls.from_string(str(value) if value else None)

    @classmethod
    def from_string(
        cls: type[Self],
        value: str | None,
    ) -> Self | None:
        """
        Convert a string value into an Enum instance

        If value is None or empty None is returned
        """
        if not value:
            return None

        try:
            return cls[value.replace(" ", "_").upper()]
        except KeyError:
            raise InvalidArgument(
                f"Invalid argument {value}. "
                f"Allowed values are {','.join(e.name for e in cls)}."
            ) from None

    def __str__(self) -> str:
        return self.value



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
Protocol implementations for the Greenbone Management Protocol (GMP).

In most circumstances you will want to use the :class:`GMP` class which
dynamically selects the supported GMP protocol of the remote manager daemon.

If you need to use a specific GMP version, you can use the :class:`GMPv224`,
:class:`GMPv225`, :class:`GMPv226`, :class:`GMPv227` or :class:`GMPNext` classes.

* :class:`GMP` - Dynamically select supported GMP protocol of the remote manager daemon.
* :class:`GMPv224` - GMP version 22.4
* :class:`GMPv225` - GMP version 22.5
* :class:`GMPv226` - GMP version 22.6
* :class:`GMPv227` - GMP version 22.7
* :class:`GMPNext` - "next" GMP version, containing unstable features
"""

from ._gmp import GMP
from ._gmp224 import GMPv224
from ._gmp225 import GMPv225
from ._gmp226 import GMPv226
from ._gmp227 import GMPv227
from ._gmpnext import GMPNext

Gmp = GMP  # for backwards compatibility

__all__ = (
    "GMP",
    "GMPNext",
    "GMPv224",
    "GMPv225",
    "GMPv226",
    "GMPv227",
    "Gmp",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetScheduleTestMixin:
    def test_get_schedule(self):
        self.gmp.get_schedule("s1")

        self.connection.send.has_been_called_with(
            b'<get_schedules schedule_id="s1"/>'
        )

        self.gmp.get_schedule(schedule_id="s1")

        self.connection.send.has_been_called_with(
            b'<get_schedules schedule_id="s1"/>'
        )

    def test_get_schedule_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_schedule(schedule_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_schedule("")

    def test_get_schedules_with_tasks(self):
        self.gmp.get_schedule(schedule_id="s1", tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_schedules schedule_id="s1" tasks="1"/>'
        )

        self.gmp.get_schedule(schedule_id="s1", tasks=False)

        self.connection.send.has_been_called_with(
            b'<get_schedules schedule_id="s1" tasks="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Results


class ResultsTestCase(unittest.TestCase):
    def test_get_result(self):
        request = Results.get_result("result_id")
        self.assertEqual(
            bytes(request),
            b'<get_results result_id="result_id" details="1"/>',
        )

    def test_get_result_missing_result_id(self):
        with self.assertRaises(RequiredArgument):
            Results.get_result(None)

        with self.assertRaises(RequiredArgument):
            Results.get_result("")

    def test_get_results(self):
        request = Results.get_results()
        self.assertEqual(
            bytes(request),
            b"<get_results/>",
        )

    def test_get_results_with_filter_string(self):
        request = Results.get_results(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_results filter="filter_string"/>',
        )

    def test_get_results_with_filter_id(self):
        request = Results.get_results(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_results filt_id="filter_id"/>',
        )

    def test_get_results_with_task_id(self):
        request = Results.get_results(task_id="task_id")
        self.assertEqual(
            bytes(request),
            b'<get_results task_id="task_id"/>',
        )

    def test_get_results_with_note_details(self):
        request = Results.get_results(note_details=True)
        self.assertEqual(
            bytes(request),
            b'<get_results note_details="1"/>',
        )

        request = Results.get_results(note_details=False)
        self.assertEqual(
            bytes(request),
            b'<get_results note_details="0"/>',
        )

    def test_get_results_with_override_details(self):
        request = Results.get_results(override_details=True)
        self.assertEqual(
            bytes(request),
            b'<get_results override_details="1"/>',
        )

        request = Results.get_results(override_details=False)
        self.assertEqual(
            bytes(request),
            b'<get_results override_details="0"/>',
        )

    def test_get_results_with_details(self):
        request = Results.get_results(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_results details="1"/>',
        )

        request = Results.get_results(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_results details="0"/>',
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreatePortListTestMixin:
    def test_create_port_list_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name=None, port_range="T:1-1234")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name="", port_range="T:1-1234")

    def test_create_port_list_missing_port_range(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name="foo", port_range=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name="foo", port_range="")

    def test_create_port_list(self):
        self.gmp.create_port_list(name="foo", port_range="T:1-1234")

        self.connection.send.has_been_called_with(
            b"<create_port_list>"
            b"<name>foo</name>"
            b"<port_range>T:1-1234</port_range>"
            b"</create_port_list>"
        )

    def test_create_port_list_with_comment(self):
        self.gmp.create_port_list(
            name="foo", port_range="T:1-1234", comment="lorem"
        )

        self.connection.send.has_been_called_with(
            b"<create_port_list>"
            b"<name>foo</name>"
            b"<port_range>T:1-1234</port_range>"
            b"<comment>lorem</comment>"
            b"</create_port_list>"
        )



# SPDX-FileCopyrightText: 2018-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpGetTaskTestMixin:
    def test_get_task(self):
        self.gmp.get_task("a1")

        self.connection.send.has_been_called_with(
            b'<get_tasks task_id="a1" usage_type="scan" details="1"/>'
        )

    def test_fail_without_task_id(self):
        with self.assertRaises(GvmError):
            self.gmp.get_task(None)

        with self.assertRaises(GvmError):
            self.gmp.get_task("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Vulnerabilities:
    @staticmethod
    def get_vulnerabilities(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
    ) -> Request:
        """Request a list of vulnerabilities

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
        """
        cmd = XmlCommand("get_vulns")
        cmd.add_filter(filter_string, filter_id)
        return cmd

    @classmethod
    def get_vulnerability(cls, vulnerability_id: EntityID) -> Request:
        """Request a single vulnerability

        Args:
            vulnerability_id: ID of an existing vulnerability
        """
        if not vulnerability_id:
            raise RequiredArgument(
                function=cls.get_vulnerability.__name__,
                argument="vulnerability_id",
            )

        cmd = XmlCommand("get_vulns")
        cmd.set_attribute("vuln_id", str(vulnerability_id))
        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.errors import RequiredArgument
from gvm.protocols.gmp._gmp226 import ReportConfigParameter


class GMPModifyReportConfigTestMixin:
    def test_modify_report_config(self):
        self.gmp.modify_report_config("report_config_id")

        self.connection.send.has_been_called_with(
            b'<modify_report_config report_config_id="report_config_id"/>',
        )

    def test_modify_report_config_with_name(self):
        self.gmp.modify_report_config("report_config_id", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_report_config report_config_id="report_config_id">'
            b"<name>foo</name>"
            b"</modify_report_config>",
        )

    def test_modify_report_config_with_comment(self):
        self.gmp.modify_report_config("report_config_id", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_report_config report_config_id="report_config_id">'
            b"<comment>foo</comment>"
            b"</modify_report_config>",
        )

    def test_modify_report_config_with_params(self):
        self.gmp.modify_report_config(
            "report_config_id",
            params=[
                ReportConfigParameter("name", "value"),
                ReportConfigParameter("name2", "value2", use_default=True),
                ReportConfigParameter("name3", use_default=True),
            ],
        )

        self.connection.send.has_been_called_with(
            b'<modify_report_config report_config_id="report_config_id">'
            b'<param><name>name</name><value use_default="0">value</value></param>'
            b'<param><name>name2</name><value use_default="1"/></param>'
            b'<param><name>name3</name><value use_default="1"/></param>'
            b"</modify_report_config>",
        )

    def test_modify_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_config(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_config("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .vulnerabilities import (
    GmpGetVulnerabilitiesTestMixin,
    GmpGetVulnerabilityTestMixin,
)


class Gmpv224GetVulnerabilityTestCase(
    GmpGetVulnerabilityTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetVulnerabilitiesTestCase(
    GmpGetVulnerabilitiesTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetUserTestMixin:
    def test_get_user(self):
        self.gmp.get_user("u1")

        self.connection.send.has_been_called_with(b'<get_users user_id="u1"/>')

        self.gmp.get_user(user_id="u1")

        self.connection.send.has_been_called_with(b'<get_users user_id="u1"/>')

    def test_get_user_missing_user_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_user(user_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_user("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_target import GmpCloneTargetTestMixin
from .test_create_target import GmpCreateTargetTestMixin
from .test_delete_target import GmpDeleteTargetTestMixin
from .test_get_target import GmpGetTargetTestMixin
from .test_get_targets import GmpGetTargetsTestMixin
from .test_modify_target import GmpModifyTargetTestMixin

__all__ = (
    "GmpCloneTargetTestMixin",
    "GmpCreateTargetTestMixin",
    "GmpDeleteTargetTestMixin",
    "GmpGetTargetTestMixin",
    "GmpGetTargetsTestMixin",
    "GmpModifyTargetTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class CallableMock:
    def __init__(self, name):
        self.name = name
        self.calls = []
        self.result = None

    def __call__(self, *args, **kwargs):
        self.calls.append({"args": args, "kwargs": kwargs})

        return self.result

    def return_value(self, value):
        self.result = value

    def has_not_been_called(self):
        assert len(self.calls) == 0, f"{self.name} has been called."

    def has_been_called(self):
        assert len(self.calls) > 0, f"{self.name} has not been called."

    def has_been_called_times(self, times):
        assert len(self.calls) == times, (
            f"{self.name} has not been called {times} times."
        )

    def has_been_called_with(self, *args, **kwargs):
        if len(self.calls) == 0:
            assert False

        lastcall = self.calls[-1]

        resp = (
            "Expected arguments {eargs} {ekwargs} of {name} do not match. "
            "Received: {rargs} {rkwargs}"
        )

        # not sure if this is correct
        assert lastcall["args"] == args and lastcall["kwargs"] == kwargs, (
            resp.format(
                name=self.name,
                eargs=args,
                ekwargs=kwargs,
                rargs=lastcall["args"],
                rkwargs=lastcall["kwargs"],
            )
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from ._connection import DEFAULT_TIMEOUT, GvmConnection
from ._debug import DebugConnection
from ._ssh import (
    DEFAULT_HOSTNAME,
    DEFAULT_KNOWN_HOSTS_FILE,
    DEFAULT_SSH_PASSWORD,
    DEFAULT_SSH_PORT,
    DEFAULT_SSH_USERNAME,
    SSHConnection,
)
from ._tls import DEFAULT_GVM_PORT, TLSConnection
from ._unix import DEFAULT_UNIX_SOCKET_PATH, UnixSocketConnection

__all__ = (
    "DEFAULT_GVM_PORT",
    "DEFAULT_HOSTNAME",
    "DEFAULT_KNOWN_HOSTS_FILE",
    "DEFAULT_SSH_PASSWORD",
    "DEFAULT_SSH_PORT",
    "DEFAULT_SSH_USERNAME",
    "DEFAULT_TIMEOUT",
    "DEFAULT_UNIX_SOCKET_PATH",
    "DebugConnection",
    "GvmConnection",
    "SSHConnection",
    "TLSConnection",
    "UnixSocketConnection",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import SecInfo


class SecInfoTestCase(unittest.TestCase):
    def test_get_info(self):
        request = SecInfo.get_info("info_id", "NVT")
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="info_id" type="NVT" details="1"/>',
        )

    def test_get_info_missing_info_id(self):
        with self.assertRaises(RequiredArgument):
            SecInfo.get_info(None, "NVT")

        with self.assertRaises(RequiredArgument):
            SecInfo.get_info("", "NVT")

    def test_get_info_missing_info_type(self):
        with self.assertRaises(RequiredArgument):
            SecInfo.get_info("info_id", None)

        with self.assertRaises(RequiredArgument):
            SecInfo.get_info("info_id", "")

    def test_get_info_invalid_info_type(self):
        with self.assertRaises(InvalidArgument):
            SecInfo.get_info("info_id", "invalid")

    def test_get_info_list(self):
        request = SecInfo.get_info_list("nvt")
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT"/>',
        )

    def test_get_info_list_with_details(self):
        request = SecInfo.get_info_list("nvt", details=True)
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" details="1"/>',
        )

        request = SecInfo.get_info_list("nvt", details=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" details="0"/>',
        )

    def test_get_info_list_with_filter_string(self):
        request = SecInfo.get_info_list("nvt", filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" filter="filter_string"/>',
        )

    def test_get_info_list_with_filter_id(self):
        request = SecInfo.get_info_list("nvt", filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" filt_id="filter_id"/>',
        )

    def test_get_info_list_with_name(self):
        request = SecInfo.get_info_list("nvt", name="name")
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" name="name"/>',
        )

    def test_get_info_list_missing_info_type(self):
        with self.assertRaises(RequiredArgument):
            SecInfo.get_info_list(None)

        with self.assertRaises(RequiredArgument):
            SecInfo.get_info_list("")

    def test_get_info_list_invalid_info_type(self):
        with self.assertRaises(InvalidArgument):
            SecInfo.get_info_list("invalid")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .trashcan import (
    GmpEmptyTrashcanTestMixin,
    GmpRestoreFromTrashcanTestMixin,
)


class Gmpv224EmptyTrashcanTestCase(GmpEmptyTrashcanTestMixin, Gmpv224TestCase):
    pass


class Gmpv224RestoreFromTrashcanTestCase(
    GmpRestoreFromTrashcanTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .policies import (
    GmpClonePolicyTestMixin,
    GmpCreatePolicyTestMixin,
    GmpDeletePolicyTestMixin,
    GmpGetPoliciesTestMixin,
    GmpGetPolicyTestMixin,
    GmpImportPolicyTestMixin,
    GmpModifyPolicySetCommentTestMixin,
    GmpModifyPolicySetFamilySelectionTestMixin,
    GmpModifyPolicySetNameTestMixin,
    GmpModifyPolicySetNvtPreferenceTestMixin,
    GmpModifyPolicySetNvtSelectionTestMixin,
    GmpModifyPolicySetScannerPreferenceTestMixin,
)


class Gmpv224ClonePolicyTestCase(GmpClonePolicyTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreatePolicyTestCase(GmpCreatePolicyTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeletePolicyTestCase(GmpDeletePolicyTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetPolicyTestCase(GmpGetPolicyTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetPoliciesTestCase(GmpGetPoliciesTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ImportPolicyTestCase(GmpImportPolicyTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyPolicySetCommentTestCase(
    GmpModifyPolicySetCommentTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPolicySetFamilySelectionTestCase(
    GmpModifyPolicySetFamilySelectionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPolicySetNvtSelectionTestCase(
    GmpModifyPolicySetNvtSelectionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPolicySetNameTestCase(
    GmpModifyPolicySetNameTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPolicySetNvtPreferenceTestCase(
    GmpModifyPolicySetNvtPreferenceTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPolicySetScannerPreferenceTestCase(
    GmpModifyPolicySetScannerPreferenceTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_scanner import GmpCloneScannerTestMixin
from .test_create_scanner import GmpCreateScannerTestMixin
from .test_delete_scanner import GmpDeleteScannerTestMixin
from .test_get_scanner import GmpGetScannerTestMixin
from .test_get_scanners import GmpGetScannersTestMixin
from .test_modify_scanner import GmpModifyScannerTestMixin
from .test_verify_scanner import GmpVerifyScannerTestMixin

__all__ = (
    "GmpCloneScannerTestMixin",
    "GmpCreateScannerTestMixin",
    "GmpDeleteScannerTestMixin",
    "GmpGetScannerTestMixin",
    "GmpGetScannersTestMixin",
    "GmpModifyScannerTestMixin",
    "GmpVerifyScannerTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetReportsTestMixin:
    def test_get_reports(self):
        self.gmp.get_reports()

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan"/>'
        )

    def test_get_reports_with_filter_string(self):
        self.gmp.get_reports(filter_string="name=foo", ignore_pagination=1)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" report_filter="name=foo" ignore_pagination="1"/>'
        )

    def test_get_reports_with_filter_id(self):
        self.gmp.get_reports(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" report_filt_id="f1"/>'
        )

    def test_get_reports_without_note_details(self):
        self.gmp.get_reports(note_details=False)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" note_details="0"/>'
        )

    def test_get_reports_with_note_details(self):
        self.gmp.get_reports(note_details=True, ignore_pagination=False)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" note_details="1" ignore_pagination="0"/>'
        )

    def test_get_reports_without_override_details(self):
        self.gmp.get_reports(override_details=False)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" override_details="0"/>'
        )

    def test_get_reports_with_override_details(self):
        self.gmp.get_reports(override_details=True)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" override_details="1"/>'
        )

    def test_get_reports_with_details(self):
        self.gmp.get_reports(details=True)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" details="1"/>'
        )

    def test_get_reports_without_details(self):
        self.gmp.get_reports(details=False)

        self.connection.send.has_been_called_with(
            b'<get_reports usage_type="scan" details="0"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetReportFormatsTestMixin:
    def test_get_report_formats(self):
        self.gmp.get_report_formats()

        self.connection.send.has_been_called_with(b"<get_report_formats/>")

    def test_get_report_formats_with_filter_string(self):
        self.gmp.get_report_formats(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_report_formats filter="foo=bar"/>'
        )

    def test_get_report_formats_with_filter_id(self):
        self.gmp.get_report_formats(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_report_formats filt_id="f1"/>'
        )

    def test_get_report_formats_with_trash(self):
        self.gmp.get_report_formats(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_report_formats trash="1"/>'
        )

        self.gmp.get_report_formats(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_report_formats trash="0"/>'
        )

    def test_get_report_formats_with_details(self):
        self.gmp.get_report_formats(details=True)

        self.connection.send.has_been_called_with(
            b'<get_report_formats details="1"/>'
        )

        self.gmp.get_report_formats(details=False)

        self.connection.send.has_been_called_with(
            b'<get_report_formats details="0"/>'
        )

    def test_get_report_formats_with_alerts(self):
        self.gmp.get_report_formats(alerts=True)

        self.connection.send.has_been_called_with(
            b'<get_report_formats alerts="1"/>'
        )

        self.gmp.get_report_formats(alerts=False)

        self.connection.send.has_been_called_with(
            b'<get_report_formats alerts="0"/>'
        )

    def test_get_report_formats_with_params(self):
        self.gmp.get_report_formats(params=True)

        self.connection.send.has_been_called_with(
            b'<get_report_formats params="1"/>'
        )

        self.gmp.get_report_formats(params=False)

        self.connection.send.has_been_called_with(
            b'<get_report_formats params="0"/>'
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpVerifyCredentialStoreTestMixin:
    def test_verify_credential_store(self):
        self.gmp.verify_credential_store(credential_store_id="cs1")

        self.connection.send.has_been_called_with(
            b'<verify_credential_store credential_store_id="cs1"/>'
        )

    def test_verify_credential_store_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.verify_credential_store(credential_store_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.verify_credential_store(credential_store_id="")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import GvmError
from gvm.xml import parse_xml


class ParseXmlTestCase(unittest.TestCase):
    def test_missing_closing_tag(self):
        with self.assertRaises(GvmError):
            parse_xml("<foo>")

    def test_invalid_tag(self):
        with self.assertRaises(GvmError):
            parse_xml("<foo&bar/>")

    def test_xml_bomb(self):
        xml = (
            "<!DOCTYPE xmlbomb ["
            '<!ENTITY a "1234567890" >'
            '<!ENTITY b "&a;&a;&a;&a;&a;&a;&a;&a;">'
            '<!ENTITY c "&b;&b;&b;&b;&b;&b;&b;&b;">'
            '<!ENTITY d "&c;&c;&c;&c;&c;&c;&c;&c;">'
            "]>"
            "<bomb>&c;</bomb>"
        )
        with self.assertRaises(GvmError):
            parse_xml(xml)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_cert_bund_advisories import GmpGetCertBundListTestMixin
from .test_get_cert_bund_advisory import GmpGetCertBundTestMixin
from .test_get_cpe import GmpGetCpeTestMixin
from .test_get_cpes import GmpGetCpeListTestMixin
from .test_get_cve import GmpGetCveTestMixin
from .test_get_cves import GmpGetCveListTestMixin
from .test_get_dfn_cert_advisories import GmpGetDfnCertListTestMixin
from .test_get_dfn_cert_advisory import GmpGetDfnCertTestMixin
from .test_get_info import GmpGetInfoTestMixin
from .test_get_info_list import GmpGetInfoListTestMixin
from .test_get_nvt import GmpGetNvtTestMixin
from .test_get_nvt_families import GmpGetNvtFamiliesTestMixin
from .test_get_nvt_preference import GmpGetNvtPreferenceTestMixin
from .test_get_nvt_preferences import GmpGetNvtPreferencesTestMixin
from .test_get_nvts import GmpGetNvtListTestMixin
from .test_get_oval_definition import GmpGetOvalDefTestMixin
from .test_get_oval_definitions import GmpGetOvalDefListTestMixin
from .test_get_scan_config_nvt import GmpGetScanConfigNvtTestMixin
from .test_get_scan_config_nvts import GmpGetScanConfigNvtsTestMixin

__all__ = (
    "GmpGetCertBundListTestMixin",
    "GmpGetCertBundTestMixin",
    "GmpGetCpeListTestMixin",
    "GmpGetCpeTestMixin",
    "GmpGetCveListTestMixin",
    "GmpGetCveTestMixin",
    "GmpGetDfnCertListTestMixin",
    "GmpGetDfnCertTestMixin",
    "GmpGetInfoListTestMixin",
    "GmpGetInfoTestMixin",
    "GmpGetNvtFamiliesTestMixin",
    "GmpGetNvtListTestMixin",
    "GmpGetNvtPreferenceTestMixin",
    "GmpGetNvtPreferencesTestMixin",
    "GmpGetNvtTestMixin",
    "GmpGetOvalDefListTestMixin",
    "GmpGetOvalDefTestMixin",
    "GmpGetScanConfigNvtTestMixin",
    "GmpGetScanConfigNvtsTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Module for GVM errors
"""


class GvmError(Exception):
    """An exception for gvm errors

    Base class for all exceptions originating in python-gvm.
    """

    def __init__(self, message: str | None, *args):
        super().__init__(message, *args)
        self.message = message

    def __repr__(self):
        return f'<{self.__class__.__name__} message="{self.message}">'

    def __str__(self):
        return self.message


class GvmClientError(GvmError):
    """An exception for gvm client errors

    Base class for all exceptions originating in python-gvm.
    """


class GvmServerError(GvmError):
    """An exception for gvm server errors

    Derives from :py:class:`GvmError`

    Arguments:
        status:  The HTTP response status
        message: Error message to be displayed. Takes precedence over argument
            and function
    """

    def __init__(self, status: str | None = None, message: str | None = None):
        super().__init__(message, status)
        self.status = status

    def __str__(self):
        return f"Server Error {self.status}. {self.message}"

    def __repr__(self):
        return (
            f'<{self.__class__.__name__} status="{self.status}"'
            f' message="{self.message}">'
        )


class GvmResponseError(GvmClientError):
    """An exception for gvm server errors

    Derives from :py:class:`GvmClientError`

    Arguments:
        status:  The HTTP response status
        message: Error message to be displayed. Takes precedence over argument
            and function
    """

    def __init__(self, status: str | None = None, message: str | None = None):
        super().__init__(message, status)
        self.status = status

    def __str__(self):
        return f"Response Error {self.status}. {self.message}"

    def __repr__(self):
        return (
            f'<{self.__class__.__name__} status="{self.status}"'
            f' message="{self.message}">'
        )


class InvalidArgument(GvmError):  # noqa: N818
    """Raised if an invalid argument/parameter is passed

    Derives from :py:class:`GvmError`

    Arguments:
        message: Error message to be displayed. Takes precedence over argument
            and function
        argument: Optional name of the invalid argument
        function: Optional name of the called function
    """

    def __init__(
        self,
        message: str | None = None,
        *,
        argument: str | None = None,
        function: str | None = None,
    ):
        super().__init__(message, argument, function)
        self.argument = argument
        self.function = function

    def __str__(self):
        if self.message:
            return self.message

        if not self.function:
            return f"Invalid argument {self.argument}"

        if not self.argument:
            return f"Invalid argument for {self.function}"

        return f"Invalid argument {self.argument} for {self.function}"


class InvalidArgumentType(GvmError):  # noqa: N818
    """Raised if a passed argument has an invalid type

    Derives from :py:class:`GvmError`

    Arguments:
        argument: Name of the invalid argument
        arg_type: The correct argument type
        function: Optional name of the called function
    """

    def __init__(
        self,
        argument: str,
        *,
        arg_type: str | None = None,
        function: str | None = None,
    ):
        super().__init__(None)
        self.argument = argument
        self.function = function
        self.arg_type = arg_type

    def __str__(self):
        if self.function:
            if self.arg_type:
                return (
                    f"In {self.function} the argument {self.argument} "
                    f"must be of type {self.arg_type}."
                )
            return (
                f"Invalid argument type for argument {self.argument} in "
                f"{self.function}."
            )
        if self.arg_type:
            return (
                f"The argument {self.argument} must be of type {self.arg_type}."
            )

        return f"Invalid argument type for argument {self.argument}."


class RequiredArgument(GvmError):  # noqa: N818
    """Raised if a required argument/parameter is missing

    Derives from :py:class:`GvmError`

    Arguments:
        message: Error message to be displayed. Takes precedence over argument
            and function.
        argument: Optional name of the required argument.
        function: Optional name of the called function.
    """

    def __init__(
        self,
        message: str | None = None,
        *,
        argument: str | None = None,
        function: str | None = None,
    ):
        super().__init__(message, argument, function)
        self.argument = argument
        self.function = function

    def __str__(self):
        if self.message:
            return self.message

        if not self.function:
            return f"Required argument {self.argument}"

        if not self.argument:
            return f"Required argument missing for {self.function}"

        return f"{self.function} requires a {self.argument} argument"



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_oci_image_target import GmpCloneOCIImageTargetTestMixin
from .test_create_oci_image_target import GmpCreateOCIImageTargetTestMixin
from .test_delete_oci_image_target import GmpDeleteOCIImageTargetTestMixin
from .test_get_oci_image_target import GmpGetOCIImageTargetTestMixin
from .test_get_oci_image_targets import GmpGetOCIImageTargetsTestMixin
from .test_modify_oci_image_target import GmpModifyOCIImageTargetTestMixin

__all__ = (
    "GmpCloneOCIImageTargetTestMixin",
    "GmpCreateOCIImageTargetTestMixin",
    "GmpDeleteOCIImageTargetTestMixin",
    "GmpGetOCIImageTargetTestMixin",
    "GmpGetOCIImageTargetsTestMixin",
    "GmpModifyOCIImageTargetTestMixin",
)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import ScannerType


class GetScannerTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            ScannerType.from_string("foo")

    def test_none_or_empty(self):
        ct = ScannerType.from_string(None)
        self.assertIsNone(ct)
        ct = ScannerType.from_string("")
        self.assertIsNone(ct)

    def test_openvas_scanner(self):
        ct = ScannerType.from_string("2")
        self.assertEqual(ct, ScannerType.OPENVAS_SCANNER_TYPE)

        ct = ScannerType.from_string("openvas")
        self.assertEqual(ct, ScannerType.OPENVAS_SCANNER_TYPE)

    def test_cve_scanner(self):
        ct = ScannerType.from_string("3")
        self.assertEqual(ct, ScannerType.CVE_SCANNER_TYPE)

        ct = ScannerType.from_string("cve")
        self.assertEqual(ct, ScannerType.CVE_SCANNER_TYPE)

    def test_gmp_scanner(self):
        with self.assertRaises(InvalidArgument):
            ScannerType.from_string("4")

        with self.assertRaises(InvalidArgument):
            ScannerType.from_string("gmp")

    def test_greenbone_sensor_scanner(self):
        ct = ScannerType.from_string("5")
        self.assertEqual(ct, ScannerType.GREENBONE_SENSOR_SCANNER_TYPE)

        ct = ScannerType.from_string("greenbone")
        self.assertEqual(ct, ScannerType.GREENBONE_SENSOR_SCANNER_TYPE)


if __name__ == "__main__":
    unittest.main()



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

"""
Greenbone Management Protocol (GMP) version 22.7
"""

from .._protocol import T
from ._gmp226 import GMPv226
from .requests.v227 import (
    EntityID,
    Scanners,
    ScannerType,
)


class GMPv227(GMPv226[T]):
    """
    A class implementing the Greenbone Management Protocol (GMP) version 22.7

    Example:

        .. code-block:: python

            from gvm.protocols.gmp import GMPv227 as GMP

            with GMP(connection) as gmp:
                resp = gmp.get_tasks()
    """

    @staticmethod
    def get_protocol_version() -> tuple[int, int]:
        return (22, 7)

    def create_scanner(  # type: ignore[override]
        self,
        name: str,
        host: str,
        port: str | int,
        scanner_type: ScannerType,
        credential_id: str,
        *,
        ca_pub: str | None = None,
        comment: str | None = None,
        relay_host: str | None = None,
        relay_port: str | int | None = None,
    ) -> T:
        """Create a new scanner

        Args:
            name: Name of the new scanner
            host: Hostname or IP address of the scanner
            port: Port of the scanner
            scanner_type: Type of the scanner
            credential_id: UUID of client certificate credential for the
                scanner
            ca_pub: Certificate of CA to verify scanner certificate
            comment: Comment for the scanner
            relay_host: Hostname or IP address of the scanner relay
            relay_port: Port of the scanner relay
        """
        return self._send_request_and_transform_response(
            Scanners.create_scanner(
                name,
                host,
                port,
                scanner_type,
                credential_id,
                ca_pub=ca_pub,
                comment=comment,
                relay_host=relay_host,
                relay_port=relay_port,
            )
        )

    def modify_scanner(  # type: ignore[override]
        self,
        scanner_id: EntityID,
        *,
        name: str | None = None,
        host: str | None = None,
        port: int | None = None,
        scanner_type: ScannerType | None = None,
        credential_id: EntityID | None = None,
        ca_pub: str | None = None,
        comment: str | None = None,
        relay_host: str | None = None,
        relay_port: str | int | None = None,
    ) -> T:
        """Modify an existing scanner

        Args:
            scanner_id: UUID of the scanner to modify
            name: New name of the scanner
            host: New hostname or IP address of the scanner
            port: New port of the scanner
            scanner_type: New type of the scanner
            credential_id: New UUID of client certificate credential for the
                scanner
            ca_pub: New certificate of CA to verify scanner certificate
            comment: New comment for the scanner
            relay_host: Hostname or IP address of the scanner relay
            relay_port: Port of the scanner relay
        """
        return self._send_request_and_transform_response(
            Scanners.modify_scanner(
                scanner_id,
                name=name,
                host=host,
                port=port,
                scanner_type=scanner_type,
                credential_id=credential_id,
                ca_pub=ca_pub,
                comment=comment,
                relay_host=relay_host,
                relay_port=relay_port,
            )
        )

    def get_scanners(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
    ) -> T:
        """Request a list of scanners

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan scanners instead
            details: Whether to include extra details like tasks using this
                scanner
        """
        return self._send_request_and_transform_response(
            Scanners.get_scanners(
                filter_string=filter_string,
                filter_id=filter_id,
                trash=trash,
                details=details,
            )
        )

    def get_scanner(self, scanner_id: EntityID) -> T:
        """Request a single scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        return self._send_request_and_transform_response(
            Scanners.get_scanner(scanner_id)
        )

    def verify_scanner(self, scanner_id: EntityID) -> T:
        """Verify an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        return self._send_request_and_transform_response(
            Scanners.verify_scanner(scanner_id)
        )

    def clone_scanner(self, scanner_id: EntityID) -> T:
        """Clone an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        return self._send_request_and_transform_response(
            Scanners.clone_scanner(scanner_id)
        )

    def delete_scanner(
        self, scanner_id: EntityID, ultimate: bool | None = False
    ) -> T:
        """Delete an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        return self._send_request_and_transform_response(
            Scanners.delete_scanner(scanner_id, ultimate=ultimate)
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from ._entity_id import EntityID
from ._version import Version

__all__ = (
    "EntityID",
    "Version",
)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.secinfo import (
    GmpGetCertBundListTestMixin,
    GmpGetCertBundTestMixin,
    GmpGetCpeListTestMixin,
    GmpGetCpeTestMixin,
    GmpGetCveListTestMixin,
    GmpGetCveTestMixin,
    GmpGetDfnCertListTestMixin,
    GmpGetDfnCertTestMixin,
    GmpGetInfoListTestMixin,
    GmpGetInfoTestMixin,
    GmpGetNvtFamiliesTestMixin,
    GmpGetNvtListTestMixin,
    GmpGetNvtTestMixin,
    GmpGetScanConfigNvtsTestMixin,
    GmpGetScanConfigNvtTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPGetCertBundTestCase(GmpGetCertBundTestMixin, GMPTestCase):
    pass


class GMPGetCpeTestCase(GmpGetCpeTestMixin, GMPTestCase):
    pass


class GMPGetCveTestCase(GmpGetCveTestMixin, GMPTestCase):
    pass


class GMPGetDfnCertCase(GmpGetDfnCertTestMixin, GMPTestCase):
    pass


class GMPGetInfoListTestCase(GmpGetInfoListTestMixin, GMPTestCase):
    pass


class GMPGetInfoTestCase(GmpGetInfoTestMixin, GMPTestCase):
    pass


class GMPGetNvtTestCase(GmpGetNvtTestMixin, GMPTestCase):
    pass


class GMPGetScanConfigNvtTestCase(GmpGetScanConfigNvtTestMixin, GMPTestCase):
    pass


class GMPGetNvtFamiliesTestCase(GmpGetNvtFamiliesTestMixin, GMPTestCase):
    pass


class GMPGetScanConfigNvtsTestCase(GmpGetScanConfigNvtsTestMixin, GMPTestCase):
    pass


class GMPGetCertBundListTestCase(GmpGetCertBundListTestMixin, GMPTestCase):
    pass


class GMPGetCpeListTestCase(GmpGetCpeListTestMixin, GMPTestCase):
    pass


class GMPGetCveListTestCase(GmpGetCveListTestMixin, GMPTestCase):
    pass


class GMPGetDfnCertListCase(GmpGetDfnCertListTestMixin, GMPTestCase):
    pass


class GMPGetNvtListTestCase(GmpGetNvtListTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetRoleTestMixin:
    def test_get_role(self):
        self.gmp.get_role("r1")

        self.connection.send.has_been_called_with(b'<get_roles role_id="r1"/>')

        self.gmp.get_role(role_id="r1")

        self.connection.send.has_been_called_with(b'<get_roles role_id="r1"/>')

    def test_get_role_missing_role_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_role(role_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_role("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from ._connection import Connection, InvalidStateError
from ._request import Request
from ._response import Response, StatusError

__all__ = (
    "Connection",
    "InvalidStateError",
    "Request",
    "Response",
    "StatusError",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.protocols.ospv1 import Osp

from .. import MockConnection


class OSPGetScannerDetailsTestCase(unittest.TestCase):
    def setUp(self):
        self.connection = MockConnection()
        self.osp = Osp(self.connection)

    def test_get_scanner_details_with_params(self):
        self.osp.get_scanner_details()

        self.connection.send.has_been_called_with(b"<get_scanner_details/>")


if __name__ == "__main__":
    unittest.main()
